@@ -6,7 +6,9 @@
 #' @keywords internal
 #' @param data a data frame
 tbl_df <- function(data) {
-  as_data_frame(data)
+  # Works in tibble < 1.5.0 too, because .name_repair will be
+  # swallowed by the ellipsis
+  as_tibble(data, .name_repair = "check_unique")
 }
 
 #' @export
@@ -27,32 +29,21 @@ auto_copy.tbl_df <- function(x, y, copy = FALSE, ...) {
   as.data.frame(y)
 }
 
-# Grouping methods ------------------------------------------------------------
-
-# These are all inherited from data.frame - see tbl-data-frame.R
-
-# Standard data frame methods --------------------------------------------------
-
-#' @export
-as.data.frame.tbl_df <- function(x, row.names = NULL, optional = FALSE, ...) {
-  as_regular_df(x)
-}
-
 # Verbs ------------------------------------------------------------------------
 
 #' @export
-arrange.tbl_df <- function(.data, ...) {
+arrange.tbl_df <- function(.data, ..., .by_group = FALSE) {
   dots <- quos(...)
-  arrange_impl(.data, dots)
+  arrange_impl(.data, dots, environment())
 }
 #' @export
-arrange_.tbl_df <- function(.data, ..., .dots = list()) {
+arrange_.tbl_df <- function(.data, ..., .dots = list(), .by_group = FALSE) {
   dots <- compat_lazy_dots(.dots, caller_env(), ...)
-  arrange_impl(.data, dots)
+  arrange_impl(.data, dots, environment())
 }
 
 #' @export
-filter.tbl_df <- function(.data, ...) {
+filter.tbl_df <- function(.data, ..., .preserve = FALSE) {
   dots <- quos(...)
   if (any(have_name(dots))) {
     bad <- dots[have_name(dots)]
@@ -61,46 +52,59 @@ filter.tbl_df <- function(.data, ...) {
     return(.data)
   }
 
-  quo <- all_exprs(!!! dots, .vectorised = TRUE)
-  filter_impl(.data, quo)
+  quo <- all_exprs(!!!dots, .vectorised = TRUE)
+  out <- filter_impl(.data, quo)
+  if (!.preserve && is_grouped_df(.data)) {
+    attr(out, "groups") <- regroup(attr(out, "groups"), environment())
+  }
+  out
 }
 #' @export
 filter_.tbl_df <- function(.data, ..., .dots = list()) {
   dots <- compat_lazy_dots(.dots, caller_env(), ...)
-  filter(.data, !!! dots)
+  filter(.data, !!!dots)
 }
 
 #' @export
-slice.tbl_df <- function(.data, ...) {
-  dots <- named_quos(...)
-  slice_impl(.data, dots)
+slice.tbl_df <- function(.data, ..., .preserve = FALSE) {
+  dots <- quos(...)
+  if (is_empty(dots)) {
+    return(.data)
+  }
+
+  quo <- quo(c(!!!dots))
+  out <- slice_impl(.data, quo)
+  if (!.preserve && is_grouped_df(.data)) {
+    attr(out, "groups") <- regroup(attr(out, "groups"), environment())
+  }
+  out
 }
 #' @export
 slice_.tbl_df <- function(.data, ..., .dots = list()) {
-  dots <- compat_lazy_dots(.dots, caller_env(), ..., .named = TRUE)
-  slice_impl(.data, dots)
+  dots <- compat_lazy_dots(.dots, caller_env(), ...)
+  slice(.data, !!!dots)
 }
 
 #' @export
 mutate.tbl_df <- function(.data, ...) {
-  dots <- named_quos(...)
-  mutate_impl(.data, dots)
+  dots <- quos(..., .named = TRUE)
+  mutate_impl(.data, dots, caller_env())
 }
 #' @export
 mutate_.tbl_df <- function(.data, ..., .dots = list()) {
   dots <- compat_lazy_dots(.dots, caller_env(), ..., .named = TRUE)
-  mutate_impl(.data, dots)
+  mutate_impl(.data, dots, caller_env())
 }
 
 #' @export
 summarise.tbl_df <- function(.data, ...) {
-  dots <- named_quos(...)
-  summarise_impl(.data, dots)
+  dots <- quos(..., .named = TRUE)
+  summarise_impl(.data, dots, environment(), caller_env())
 }
 #' @export
 summarise_.tbl_df <- function(.data, ..., .dots = list()) {
   dots <- compat_lazy_dots(.dots, caller_env(), ..., .named = TRUE)
-  summarise_impl(.data, dots)
+  summarise_impl(.data, dots, environment(), caller_env())
 }
 
 # Joins ------------------------------------------------------------------------
@@ -116,7 +120,7 @@ summarise_.tbl_df <- function(.data, ..., .dots = list()) {
 #'   Use `"never"` to always treat two `NA` or `NaN` values as
 #'   different, like joins for database sources, similarly to
 #'   `merge(incomparables = FALSE)`.
-#'   The default,`"na"`, always treats two `NA` or `NaN` values as equal, like [merge()].
+#'   The default, `"na"`, always treats two `NA` or `NaN` values as equal, like [merge()].
 #'   Users and package authors can change the default behavior by calling
 #'   `pkgconfig::set_config("dplyr::na_matches" = "never")`.
 #' @examples
@@ -152,25 +156,72 @@ check_na_matches <- function(na_matches) {
 inner_join.tbl_df <- function(x, y, by = NULL, copy = FALSE,
                               suffix = c(".x", ".y"), ...,
                               na_matches = pkgconfig::get_config("dplyr::na_matches")) {
+  check_valid_names(tbl_vars(x))
+  check_valid_names(tbl_vars(y))
   by <- common_by(by, x, y)
   suffix <- check_suffix(suffix)
+  na_matches <- check_na_matches(na_matches)
 
   y <- auto_copy(x, y, copy = copy)
 
-  inner_join_impl(x, y, by$x, by$y, suffix$x, suffix$y, check_na_matches(na_matches))
+  vars <- join_vars(tbl_vars(x), tbl_vars(y), by, suffix)
+  by_x <- vars$idx$x$by
+  by_y <- vars$idx$y$by
+  aux_x <- vars$idx$x$aux
+  aux_y <- vars$idx$y$aux
+
+  out <- inner_join_impl(x, y, by_x, by_y, aux_x, aux_y, na_matches, environment())
+  names(out) <- vars$alias
+
+  reconstruct_join(out, x, vars)
 }
 
+#' @export
+#' @rdname join.tbl_df
+nest_join.tbl_df <- function(x, y, by = NULL, copy = FALSE, keep = FALSE, name = NULL, ...) {
+  name_var <- name %||% expr_name(enexpr(y))
+  check_valid_names(tbl_vars(x))
+  check_valid_names(tbl_vars(y))
+  by <- common_by(by, x, y)
+
+  y <- auto_copy(x, y, copy = copy)
+
+  vars <- join_vars(tbl_vars(x), tbl_vars(y), by)
+  by_x <- vars$idx$x$by
+  by_y <- vars$idx$y$by
+  aux_y <- vars$idx$y$aux
+  if (keep) {
+    aux_y <- c(by_y, aux_y)
+  }
+
+  out <- nest_join_impl(x, y, by_x, by_y, aux_y, name_var, environment())
+  out
+}
+
+
 #' @export
 #' @rdname join.tbl_df
 left_join.tbl_df <- function(x, y, by = NULL, copy = FALSE,
                              suffix = c(".x", ".y"), ...,
                              na_matches = pkgconfig::get_config("dplyr::na_matches")) {
+  check_valid_names(tbl_vars(x))
+  check_valid_names(tbl_vars(y))
   by <- common_by(by, x, y)
   suffix <- check_suffix(suffix)
+  na_matches <- check_na_matches(na_matches)
 
   y <- auto_copy(x, y, copy = copy)
 
-  left_join_impl(x, y, by$x, by$y, suffix$x, suffix$y, check_na_matches(na_matches))
+  vars <- join_vars(tbl_vars(x), tbl_vars(y), by, suffix)
+  by_x <- vars$idx$x$by
+  by_y <- vars$idx$y$by
+  aux_x <- vars$idx$x$aux
+  aux_y <- vars$idx$y$aux
+
+  out <- left_join_impl(x, y, by_x, by_y, aux_x, aux_y, na_matches, environment())
+  names(out) <- vars$alias
+
+  reconstruct_join(out, x, vars)
 }
 
 #' @export
@@ -178,11 +229,24 @@ left_join.tbl_df <- function(x, y, by = NULL, copy = FALSE,
 right_join.tbl_df <- function(x, y, by = NULL, copy = FALSE,
                               suffix = c(".x", ".y"), ...,
                               na_matches = pkgconfig::get_config("dplyr::na_matches")) {
+  check_valid_names(tbl_vars(x))
+  check_valid_names(tbl_vars(y))
   by <- common_by(by, x, y)
   suffix <- check_suffix(suffix)
+  na_matches <- check_na_matches(na_matches)
 
   y <- auto_copy(x, y, copy = copy)
-  right_join_impl(x, y, by$x, by$y, suffix$x, suffix$y, check_na_matches(na_matches))
+
+  vars <- join_vars(tbl_vars(x), tbl_vars(y), by, suffix)
+  by_x <- vars$idx$x$by
+  by_y <- vars$idx$y$by
+  aux_x <- vars$idx$x$aux
+  aux_y <- vars$idx$y$aux
+
+  out <- right_join_impl(x, y, by_x, by_y, aux_x, aux_y, na_matches, environment())
+  names(out) <- vars$alias
+
+  reconstruct_join(out, x, vars)
 }
 
 #' @export
@@ -190,39 +254,73 @@ right_join.tbl_df <- function(x, y, by = NULL, copy = FALSE,
 full_join.tbl_df <- function(x, y, by = NULL, copy = FALSE,
                              suffix = c(".x", ".y"), ...,
                              na_matches = pkgconfig::get_config("dplyr::na_matches")) {
+  check_valid_names(tbl_vars(x))
+  check_valid_names(tbl_vars(y))
   by <- common_by(by, x, y)
   suffix <- check_suffix(suffix)
+  na_matches <- check_na_matches(na_matches)
 
   y <- auto_copy(x, y, copy = copy)
-  full_join_impl(x, y, by$x, by$y, suffix$x, suffix$y, check_na_matches(na_matches))
+
+  vars <- join_vars(tbl_vars(x), tbl_vars(y), by, suffix)
+  by_x <- vars$idx$x$by
+  by_y <- vars$idx$y$by
+  aux_x <- vars$idx$x$aux
+  aux_y <- vars$idx$y$aux
+
+  out <- full_join_impl(x, y, by_x, by_y, aux_x, aux_y, na_matches, environment())
+  names(out) <- vars$alias
+
+  reconstruct_join(out, x, vars)
 }
 
 #' @export
 #' @rdname join.tbl_df
 semi_join.tbl_df <- function(x, y, by = NULL, copy = FALSE, ...,
                              na_matches = pkgconfig::get_config("dplyr::na_matches")) {
+  check_valid_names(tbl_vars(x), warn_only = TRUE)
+  check_valid_names(tbl_vars(y), warn_only = TRUE)
+
   by <- common_by(by, x, y)
   y <- auto_copy(x, y, copy = copy)
-  semi_join_impl(x, y, by$x, by$y, check_na_matches(na_matches))
+  out <- semi_join_impl(x, y, by$x, by$y, check_na_matches(na_matches), environment())
+  if (is_grouped_df(x)) {
+    out <- grouped_df_impl(out, group_vars(x), group_drops(x))
+  }
+  out
 }
 
 #' @export
 #' @rdname join.tbl_df
 anti_join.tbl_df <- function(x, y, by = NULL, copy = FALSE, ...,
                              na_matches = pkgconfig::get_config("dplyr::na_matches")) {
+  check_valid_names(tbl_vars(x), warn_only = TRUE)
+  check_valid_names(tbl_vars(y), warn_only = TRUE)
+
   by <- common_by(by, x, y)
   y <- auto_copy(x, y, copy = copy)
-  anti_join_impl(x, y, by$x, by$y, check_na_matches(na_matches))
+  out <- anti_join_impl(x, y, by$x, by$y, check_na_matches(na_matches), environment())
+  if (is_grouped_df(x)) {
+    out <- grouped_df_impl(out, group_vars(x), group_drops(x))
+  }
+  out
+}
+
+reconstruct_join <- function(out, x, vars) {
+  if (is_grouped_df(x)) {
+    groups_in_old <- match(group_vars(x), tbl_vars(x))
+    groups_in_alias <- match(groups_in_old, vars$x)
+    out <- grouped_df_impl(out, vars$alias[groups_in_alias], group_drops(x))
+  }
+  out
 }
 
 
 # Set operations ---------------------------------------------------------------
 
 #' @export
-distinct.tbl_df <- function(.data, ...) {
-  tbl_df(NextMethod())
-}
+# Can't use NextMethod() in R 3.1, r-lib/rlang#486
+distinct.tbl_df <- distinct.data.frame
 #' @export
-distinct_.tbl_df <- function(.data, ..., .dots = list()) {
-  tbl_df(NextMethod())
-}
+# Can't use NextMethod() in R 3.1, r-lib/rlang#486
+distinct_.tbl_df <- distinct_.data.frame
