@@ -40,13 +40,13 @@ test_that("summarise can refer to variables that were just created (#138)", {
 })
 
 test_that("summarise can refer to factor variables that were just created (#2217)", {
-  df <- data_frame(a = 1:3) %>%
+  df <- tibble(a = 1:3) %>%
     group_by(a)
   res <- df %>%
     summarise(f = factor(if_else(a <= 1, "a", "b")), g = (f == "a"))
   expect_equal(
     res,
-    data_frame(a = 1:3, f = factor(c("a", "b", "b")), g = c(TRUE, FALSE, FALSE))
+    tibble(a = 1:3, f = factor(c("a", "b", "b")), g = c(TRUE, FALSE, FALSE))
   )
 })
 
@@ -60,7 +60,7 @@ test_that("summarise refuses to modify grouping variable (#143)", {
 })
 
 test_that("summarise gives proper errors (#153)", {
-  df <- data_frame(
+  df <- tibble(
     x = 1,
     y = c(1, 2, 2),
     z = runif(3)
@@ -208,13 +208,13 @@ test_that("summarise propagate attributes (#194)", {
 })
 
 test_that("summarise strips names, but only if grouped (#2231, #2675)", {
-  data <- data_frame(a = 1:3) %>% summarise(b = setNames(nm = a[[1]]))
+  data <- tibble(a = 1:3) %>% summarise(b = setNames(nm = a[[1]]))
   expect_equal(names(data$b), "1")
 
-  data <- data_frame(a = 1:3) %>% rowwise() %>% summarise(b = setNames(nm = a))
+  data <- tibble(a = 1:3) %>% rowwise() %>% summarise(b = setNames(nm = a))
   expect_null(names(data$b))
 
-  data <- data_frame(a = c(1, 1, 2)) %>% group_by(a) %>% summarise(b = setNames(nm = a[[1]]))
+  data <- tibble(a = c(1, 1, 2)) %>% group_by(a) %>% summarise(b = setNames(nm = a[[1]]))
   expect_null(names(data$b))
 })
 
@@ -230,9 +230,7 @@ test_that("summarise fails on missing variables when grouping (#2223)", {
 
 test_that("n() does not accept arguments", {
   expect_error(
-    summarise(group_by(mtcars, cyl), n(hp)),
-    "`n()` does not take arguments",
-    fixed = TRUE
+    summarise(group_by(mtcars, cyl), n(hp))
   )
 })
 
@@ -270,9 +268,9 @@ test_that("summarise can use newly created variable more than once", {
   }
 })
 
-test_that("summarise creates an empty data frame when no parameters are used", {
+test_that("summarise creates an empty data frame with one row when no parameters are used", {
   res <- summarise(mtcars)
-  expect_equal(res, data.frame())
+  expect_equal(nrow(res), 1L)
 })
 
 test_that("summarise works with zero-row data frames", {
@@ -312,14 +310,14 @@ test_that("summarise checks outputs (#300)", {
   )
 })
 
-test_that("comment attribute is white listed (#346)", {
+test_that("comment attribute is allowed (#346)", {
   test <- data.frame(A = c(1, 1, 0, 0), B = c(2, 2, 3, 3))
   comment(test$B) <- "2nd Var"
   res <- group_by(test, A)
   expect_equal(comment(res$B), "2nd Var")
 })
 
-test_that("AsIs class is white listed (#453)", {
+test_that("AsIs class is allowed (#453)", {
   test <- data.frame(A = c(1, 1, 0, 0), B = I(c(2, 2, 3, 3)))
   res <- group_by(test, B)
   expect_equal(res$B, test$B)
@@ -402,7 +400,7 @@ test_that("LazySubset is not confused about input data size (#452)", {
 })
 
 test_that("nth, first, last promote dates and times (#509)", {
-  data <- data_frame(
+  data <- tibble(
     ID = rep(letters[1:4], each = 5),
     date = Sys.Date() + 1:20,
     time = Sys.time() + 1:20,
@@ -429,7 +427,7 @@ test_that("nth, first, last promote dates and times (#509)", {
 })
 
 test_that("nth, first, last preserves factor data (#509)", {
-  dat <- data_frame(a = rep(seq(1, 20, 2), 3), b = as.ordered(a))
+  dat <- tibble(a = rep(seq(1, 20, 2), 3), b = as.ordered(a))
   dat1 <- dat %>%
     group_by(a) %>%
     summarise(
@@ -470,7 +468,7 @@ test_that("nth handle negative value (#1584) ", {
 })
 
 test_that("LazyGroupSubsets is robust about columns not from the data (#600)", {
-  foo <- data_frame(x = 1:10, y = 1:10)
+  foo <- tibble(x = 1:10, y = 1:10)
   # error messages from rlang
   expect_error(foo %>% group_by(x) %>% summarise(first_y = first(z)))
 })
@@ -553,22 +551,16 @@ test_that("summarise is not polluted by logical NA (#599)", {
 })
 
 test_that("summarise handles list output columns (#832)", {
-  df <- data_frame(x = 1:10, g = rep(1:2, each = 5))
+  df <- tibble(x = 1:10, g = rep(1:2, each = 5))
   res <- df %>% group_by(g) %>% summarise(y = list(x))
   expect_equal(res$y[[1]], 1:5)
   expect_equal(res$y[[2]], 6:10)
-  # just checking objects are not messed up internally
-  expect_equal(gp(res$y[[1]]), 0L)
-  expect_equal(gp(res$y[[2]]), 0L)
 
   res <- df %>% group_by(g) %>% summarise(y = list(x + 1))
   expect_equal(res$y[[1]], 1:5 + 1)
   expect_equal(res$y[[2]], 6:10 + 1)
-  # just checking objects are not messed up internally
-  expect_equal(gp(res$y[[1]]), 0L)
-  expect_equal(gp(res$y[[2]]), 0L)
 
-  df <- data_frame(x = 1:10, g = rep(1:2, each = 5))
+  df <- tibble(x = 1:10, g = rep(1:2, each = 5))
   res <- df %>% summarise(y = list(x))
   expect_equal(res$y[[1]], 1:10)
   res <- df %>% summarise(y = list(x + 1))
@@ -578,7 +570,7 @@ test_that("summarise handles list output columns (#832)", {
 test_that("summarise works with empty data frame (#1142)", {
   df <- data.frame()
   res <- df %>% summarise()
-  expect_equal(nrow(res), 0L)
+  expect_equal(nrow(res), 1L)
   expect_equal(length(res), 0L)
 })
 
@@ -606,7 +598,7 @@ test_that("n_distinct without arguments stops (#1957)", {
 
 test_that("hybrid evaluation does not take place for objects with a class (#1237)", {
   mean.foo <- function(x) 42
-  df <- data_frame(x = structure(1:10, class = "foo"))
+  df <- tibble(x = structure(1:10, class = "foo"))
   expect_equal(summarise(df, m = mean(x))$m[1], 42)
 
   env <- environment()
@@ -665,7 +657,7 @@ test_that("summarise correctly handles logical (#1291)", {
 })
 
 test_that("summarise correctly handles NA groups (#1261)", {
-  tmp <- data_frame(
+  tmp <- tibble(
     a = c(1, 1, 1, 2, 2),
     b1 = NA_integer_,
     b2 = NA_character_
@@ -703,20 +695,33 @@ test_that("n_distinct handles multiple columns (#1084)", {
 })
 
 test_that("hybrid max works when not used on columns (#1369)", {
-  df <- data_frame(x = 1:1000)
+  df <- tibble(x = 1:1000)
   y <- 1:10
   expect_equal(summarise(df, z = max(y))$z, 10)
   expect_equal(summarise(df, z = max(10))$z, 10)
 })
 
-test_that("min and max handle empty sets in summarise (#1481)", {
-  df <- data_frame(A = numeric())
+test_that("min and max handle empty sets in summarise (#1481, #3997)", {
+  df <- tibble(A = numeric())
   res <- df %>% summarise(Min = min(A, na.rm = TRUE), Max = max(A, na.rm = TRUE))
   expect_equal(res$Min, Inf)
   expect_equal(res$Max, -Inf)
 })
 
 test_that("lead and lag behave correctly in summarise (#1434)", {
+  res <- mtcars %>%
+    summarise(
+      n = n(),
+      leadn = lead(n),
+      lagn = lag(n),
+      leadn10 = lead(n, default = 10),
+      lagn10 = lag(n, default = 10)
+    )
+  expect_true(all(is.na(res$lagn)))
+  expect_true(all(is.na(res$leadn)))
+  expect_true(all(res$lagn10 == 10))
+  expect_true(all(res$leadn10 == 10))
+
   res <- mtcars %>%
     group_by(cyl) %>%
     summarise(
@@ -814,7 +819,7 @@ test_that("hybrid n_distinct falls back to R evaluation when needed (#1657)", {
 })
 
 test_that("summarise() correctly coerces factors with different levels (#1678)", {
-  res <- data_frame(x = 1:3) %>%
+  res <- tibble(x = 1:3) %>%
     group_by(x) %>%
     summarise(
       y = if (x == 1) "a" else "b",
@@ -826,9 +831,9 @@ test_that("summarise() correctly coerces factors with different levels (#1678)",
 })
 
 test_that("summarise handles raw columns (#1803)", {
-  df <- data_frame(a = 1:3, b = as.raw(1:3))
-  expect_equal(summarise(df, c = sum(a)), data_frame(c = 6L))
-  expect_identical(summarise(df, c = b[[1]]), data_frame(c = as.raw(1)))
+  df <- tibble(a = 1:3, b = as.raw(1:3))
+  expect_equal(summarise(df, c = sum(a)), tibble(c = 6L))
+  expect_identical(summarise(df, c = b[[1]]), tibble(c = as.raw(1)))
 })
 
 test_that("dim attribute is stripped from grouped summarise (#1918)", {
@@ -845,7 +850,7 @@ test_that("dim attribute is stripped from grouped summarise (#1918)", {
 
 test_that("typing and NAs for grouped summarise (#1839)", {
   expect_identical(
-    data_frame(id = 1L, a = NA_character_) %>%
+    tibble(id = 1L, a = NA_character_) %>%
       group_by(id) %>%
       summarise(a = a[[1]]) %>%
       .$a,
@@ -853,7 +858,7 @@ test_that("typing and NAs for grouped summarise (#1839)", {
   )
 
   expect_identical(
-    data_frame(id = 1:2, a = c(NA, "a")) %>%
+    tibble(id = 1:2, a = c(NA, "a")) %>%
       group_by(id) %>%
       summarise(a = a[[1]]) %>%
       .$a,
@@ -862,7 +867,7 @@ test_that("typing and NAs for grouped summarise (#1839)", {
 
   # Properly upgrade NA (logical) to character
   expect_identical(
-    data_frame(id = 1:2, a = 1:2) %>%
+    tibble(id = 1:2, a = 1:2) %>%
       group_by(id) %>%
       summarise(a = ifelse(all(a < 2), NA, "yes")) %>%
       .$a,
@@ -870,7 +875,7 @@ test_that("typing and NAs for grouped summarise (#1839)", {
   )
 
   expect_error(
-    data_frame(id = 1:2, a = list(1, "2")) %>%
+    tibble(id = 1:2, a = list(1, "2")) %>%
       group_by(id) %>%
       summarise(a = a[[1]]) %>%
       .$a,
@@ -879,7 +884,7 @@ test_that("typing and NAs for grouped summarise (#1839)", {
   )
 
   expect_identical(
-    data_frame(id = 1:2, a = list(1, "2")) %>%
+    tibble(id = 1:2, a = list(1, "2")) %>%
       group_by(id) %>%
       summarise(a = a[1]) %>%
       .$a,
@@ -889,7 +894,7 @@ test_that("typing and NAs for grouped summarise (#1839)", {
 
 test_that("typing and NAs for rowwise summarise (#1839)", {
   expect_identical(
-    data_frame(id = 1L, a = NA_character_) %>%
+    tibble(id = 1L, a = NA_character_) %>%
       rowwise() %>%
       summarise(a = a[[1]]) %>%
       .$a,
@@ -897,7 +902,7 @@ test_that("typing and NAs for rowwise summarise (#1839)", {
   )
 
   expect_identical(
-    data_frame(id = 1:2, a = c(NA, "a")) %>%
+    tibble(id = 1:2, a = c(NA, "a")) %>%
       rowwise() %>%
       summarise(a = a[[1]]) %>%
       .$a,
@@ -906,7 +911,7 @@ test_that("typing and NAs for rowwise summarise (#1839)", {
 
   # Properly promote NA (logical) to character
   expect_identical(
-    data_frame(id = 1:2, a = 1:2) %>%
+    tibble(id = 1:2, a = 1:2) %>%
       group_by(id) %>%
       summarise(a = ifelse(all(a < 2), NA, "yes")) %>%
       .$a,
@@ -914,7 +919,7 @@ test_that("typing and NAs for rowwise summarise (#1839)", {
   )
 
   expect_error(
-    data_frame(id = 1:2, a = list(1, "2")) %>%
+    tibble(id = 1:2, a = list(1, "2")) %>%
       rowwise() %>%
       summarise(a = a[[1]]) %>%
       .$a,
@@ -923,7 +928,7 @@ test_that("typing and NAs for rowwise summarise (#1839)", {
   )
 
   expect_error(
-    data_frame(id = 1:2, a = list(1, "2")) %>%
+    tibble(id = 1:2, a = list(1, "2")) %>%
       rowwise() %>%
       summarise(a = a[1]) %>%
       .$a,
@@ -964,7 +969,7 @@ test_that("min, max preserves ordered factor data  (#2200)", {
 })
 
 test_that("ungrouped summarise() uses summary variables correctly (#2404)", {
-  df <- tibble::as_tibble(seq(1:10))
+  df <- tibble(value = seq(1:10))
 
   out <- df %>% summarise(value = mean(value), sd = sd(value))
   expect_equal(out$value, 5.5)
@@ -972,7 +977,7 @@ test_that("ungrouped summarise() uses summary variables correctly (#2404)", {
 })
 
 test_that("proper handling of names in summarised list columns (#2231)", {
-  d <- data_frame(x = rep(1:3, 1:3), y = 1:6, names = letters[1:6])
+  d <- tibble(x = rep(1:3, 1:3), y = 1:6, names = letters[1:6])
   res <- d %>% group_by(x) %>% summarise(y = list(setNames(y, names)))
   expect_equal(names(res$y[[1]]), letters[[1]])
   expect_equal(names(res$y[[2]]), letters[2:3])
@@ -1032,3 +1037,96 @@ test_that("first() and last() can be called without dplyr loaded (#3498)", {
   expect_equal(df$y, 3L)
   expect_equal(df$z, 1L)
 })
+
+test_that("hybrid sum handles NA correctly (#3528)",{
+  d <- tibble(x = c(1L,2L,NA) )
+
+  expect_equal( summarise(d, x = sum(x, na.rm = TRUE)), tibble(x = 3L))
+  expect_equal( summarise(d, x = sum(x, na.rm = FALSE)), tibble(x = NA_integer_))
+  expect_equal( summarise(d, x = sum(x)), tibble(x = NA_integer_))
+
+  d <- tibble(x = c(TRUE, FALSE, NA) )
+  expect_equal( summarise(d, x = sum(x, na.rm = TRUE)), tibble(x = 1L))
+  expect_equal( summarise(d, x = sum(x, na.rm = FALSE)), tibble(x = NA_integer_))
+  expect_equal( summarise(d, x = sum(x)), tibble(x = NA_integer_))
+})
+
+test_that("formulas are evaluated in the right environment (#3019)", {
+  out <- mtcars %>% summarise(fn = list(rlang::as_function(~ list(~foo, environment()))))
+  out <- out$fn[[1]]()
+  expect_identical(environment(out[[1]]), out[[2]])
+})
+
+test_that("summarise correctly reconstruct group rows", {
+  d <- tibble(x = 1:4, g1 = rep(1:2, 2), g2 = 1:4) %>%
+    group_by(g1, g2) %>%
+    summarise(x = x+1)
+  expect_equal(group_rows(d), list(1:2, 3:4))
+})
+
+test_that("summarise can handle POSIXlt columns (#3854)", {
+  df <- data.frame(g=c(1,1,3))
+  df$created <- strptime(c("2014/1/1", "2014/1/2", "2014/1/2"), format = "%Y/%m/%d")
+
+  res <- df %>%
+    group_by(g) %>%
+    summarise(data = list(created))
+  expect_true(all(sapply(res$data, inherits, "POSIXlt")))
+})
+
+test_that("the data mask marks subsets as not mutable", {
+  res <- mtcars %>%
+    group_by(cyl) %>%
+    summarise(ngroup = n(), shared = is_maybe_shared(environment(), sym("ngroup")))
+  expect_true(all(res$shared))
+  expect_true(all(maybe_shared_columns(res)))
+})
+
+test_that("column_subset respects S3 local [. method (#3923)", {
+  testS3Class <- function(x, X){
+    structure(x, class = "testS3Class", X = X)
+  }
+  `[.testS3Class` <- function(x, i, ...) {
+    testS3Class(unclass(x)[i, ...], X = attr(x, "X"))
+  }
+  df <- tibble(x = rep(1:2, each = 5), y = testS3Class(1:10, X = 100))
+  res <- df %>%
+    group_by(x) %>%
+    summarise(chunk = list(y))
+  expect_equal(res$chunk[[1]], df$y[df$x == 1])
+  expect_equal(res$chunk[[1]], df$y[df$x == 1])
+
+  df$y <- testS3Class(matrix(1:20, ncol = 2), X = 200)
+  res <- df %>%
+    group_by(x) %>%
+    summarise(chunk = list(y))
+  expect_equal(res$chunk[[1]], df$y[df$x == 1, , drop = FALSE])
+  expect_equal(res$chunk[[1]], df$y[df$x == 1, , drop = FALSE])
+})
+
+test_that("tidy eval does not infloop (#4049)", {
+  df <- data.frame(x = 1:5)
+  call <- expr(length(!!quo(x)))
+  expect_identical(summarise(df, x = eval_tidy(call)), data.frame(x = 5L))
+})
+
+test_that("hybrid sum(), mean(), min(), max() treats NA and NaN correctly (#4108, #4163)", {
+  res <- data.frame(x = c(1, NaN)) %>%
+    summarise(sum = sum(x), mean = mean(x), max = max(x), min = min(x))
+  expect_true(is.nan(res$sum))
+  expect_true(is.nan(res$mean))
+  expect_true(is.nan(res$max))
+  expect_true(is.nan(res$min))
+
+  res <- data.frame(x = c(1, NaN)) %>%
+    summarise(
+      sum = sum(x, na.rm = TRUE),
+      mean = mean(x, na.rm = TRUE),
+      max = max(x, na.rm = TRUE),
+      min = min(x, na.rm = TRUE)
+    )
+  expect_equal(res$sum , 1)
+  expect_equal(res$mean, 1)
+  expect_equal(res$max , 1)
+  expect_equal(res$min , 1)
+})
